<?php

	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/session_model.php");
	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/user_model.php");
	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/stripe_model.php");
	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/pricing_model.php");

	class AdminPromotionCodesEdit extends FJF_CMD{
		
		var $sessionModel = null;
		var $userModel = null;

		function __construct(){
			$this->sessionModel = new SessionModel();
			$this->userModel = new UserModel();
		}

		function execute(){
			require_once($GLOBALS["WEB_APPLICATION_CONFIG"]["includes_path"] . "/admin_header_commands.php");

			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;

			$header["parent"] = array("promotion_codes" => "Manage Promotion Codes");
			$header["title"] = ($id ? "Edit" : "Add") . " Promotion Code";

			$record = $id ? FJF_BASE_RICH::getRecordBy("promotion_codes", $id) : new stdClass;

			if($_SERVER["REQUEST_METHOD"] == "POST"){
				
				$record->title = isset($_POST["title"]) ? trim($_POST["title"]) : "";
				$record->code = isset($_POST["code"]) && $_POST["code"] ? trim($_POST["code"]) : $this->userModel->getRandString(8);
				$record->url_title = isset($_POST["url_title"]) ? trim($_POST["url_title"]) : "";
				$record->redeem_by = isset($_POST["redeem_by"]) && $_POST["redeem_by"] ? date("Y-m-d H:i:s", strtotime($_POST["redeem_by"])) : "";
				$record->max_redemptions = isset($_POST["max_redemptions"]) && $_POST["max_redemptions"] ? intval($_POST["max_redemptions"]) : "";
				$record->type_off = isset($_POST["type_off"]) ? trim($_POST["type_off"]) : "";
				$record->discount = isset($_POST["discount"]) && $_POST["discount"] ? floatval($_POST["discount"]) : "";
				$record->tiers = isset($_POST["tiers"]) && $_POST["tiers"] ? implode(',', $_POST["tiers"]) : "";
				$record->payment_types = isset($_POST["payment_types"]) && $_POST["payment_types"] ? implode(',', $_POST["payment_types"]) : "";
				$record->free_trial = isset($_POST["free_trial"]) && $_POST["free_trial"] ? intval($_POST["free_trial"]) : "";
				$record->duration = isset($_POST["duration"]) && $_POST["duration"] ? intval($_POST["duration"]) : "";
				$record->status = isset($_POST["status"]) ? intval($_POST["status"]) : 0;

				if(!$record->title){
					$hasError = true;
					$status .= "Title is missing.<br />";
				}
				
				$record->url_title = FJF_BASE_RICH::generateURL(
					($record->url_title ? $record->url_title : $record->title),
					$id,
					"promotion_codes"
				);
				
				if(!$record->url_title){
					$hasError = true;
					$status .= "URL Parameter is missing.<br />";
				}
				
				if(!$record->discount){
					$hasError = true;
					$status .= "Discount is missing.<br />";
				}elseif($record->discount < 0){
					$hasError = true;
					$status .= "Invalid Discount format.<br />";
				}elseif($record->type_off == 'percent_off' && $record->discount > 100){
					$hasError = true;
					$status .= "Invalid Discount format.<br />";
				}
				
				if(!$record->tiers){
					$hasError = true;
					$status .= "Tiers is missing.<br />";
				}
				
				if(!$record->payment_types){
					$hasError = true;
					$status .= "Pricing Plan is missing.<br />";
				}

				if(!$hasError){
					StripeModel::setApiKey();
					$data = array(
						"id" => $record->code,
						"currency" => "usd",
						"duration" => ($record->duration ? "repeating" : "forever")
					);
					if($record->duration){
						$data["duration_in_months"] = $record->duration;
					}
					if($record->redeem_by){
						$data["redeem_by"] = strtotime($record->redeem_by);
					}
					if($record->max_redemptions){
						$data["max_redemptions"] = $record->max_redemptions;
					}
					if($record->type_off == 'amount_off'){
						$data["amount_off"] = intval($record->discount * 100);
					}elseif($record->type_off == 'percent_off'){
						$data["percent_off"] = intval($record->discount);
					}
					
					try{
						StripeModel::saveCoupon($data);
					}catch(Exception $e){
						$hasError = true;
						$status .= $e->getMessage() . "<br/> ";
					}
				}
				
				if(!$hasError){	
					if($result = FJF_BASE_RICH::saveRecord("promotion_codes", get_object_vars($record))){
						FJF_BASE::redirect("/admin/promotion_codes/");
					}else{
						$hasError = true;
						$status = "An error occured. Please try again later.";
					}
				}

			}//POST
			
			if(isset($record->tiers)) $record->tiers = explode(',', $record->tiers);
			if(isset($record->payment_types)) $record->payment_types = explode(',', $record->payment_types);

			$templateParams = array(
	      "status" => $status,
	      "has_error" => $hasError,
	      "header" => $header,
	      "record" => $record,
	      "tier_levels" => PricingModel::getTiersInfo(),
        "payment_types" => PricingModel::$paymentTypes
	     );

			$this->displayTemplate("admin_promotion_codes_edit.tpl", $templateParams);

			return true;
		}

	}
?>