<?php

  include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/session_model.php");
  include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/pagination_model.php");
  include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/sentiment_model.php");
  include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/heatmap_model.php");
  include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/user_model.php");
	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/price_model.php");

  class Dashboard extends FJF_CMD
  {
    var $sessionModel = null;
    var $paginationModel = null;
    var $sentimentModel = null;
    var $userModel = null;

    function __construct()
    {
      $this->sessionModel = new SessionModel();
			$this->sessionModel->notLoginRedirect();
			$this->sessionModel->checkLockedPage();
      $this->paginationModel = new PaginationModel();
      $this->sentimentModel = new SentimentModel();
      $this->userModel = new UserModel();
    }

    function execute()
    {
      require_once($GLOBALS["WEB_APPLICATION_CONFIG"]["app_web_path"] . "/includes/header_commands.php");
      $user = $this->sessionModel->getUser();
      $header["title"] = "Stock Dashboard";
      
      if($_SERVER["REQUEST_METHOD"] == "POST"){
        $action = isset($_POST["action"]) && $_POST["action"] ? trim($_POST["action"]) : "";
        
        if($action == 'add_stock'){
          $id = isset($_POST["id"]) && $_POST["id"] ? trim($_POST["id"]) : '';
          $data = '';
          if($id == ''){
            $data .= "Ticker is missing.";
            $hasError = true;
					}elseif(!$user->tier){
						$data = array(
							"title" => "Your account is no longer valid!",
							"message" => "If you would like to activate your subscription, please enter your credit card information with a valid credit card in User Account Settings.",
						);
						$hasError = true;
          }elseif(!$this->sentimentModel->checkStockDashboardLimit($user->id, $user->tier)){
						$data = array(
            	"title" => "Sorry, But You Have Reached Your Limit!",
            	"message" => "You have reached your maximum limit of stocks in your stock dashboard. You must remove a stock before being able to add a new one."
           	);
            $hasError = true;
          }elseif($this->sentimentModel->checkStock($user->id, $id)){
            $data = "You already have added this ticker to your dashboard.";
            $hasError = true;
          }
          if(!$hasError){
            $fields['ticker'] = $id;
            $fields['user_id'] = $user->id;
            if($this->sentimentModel->addStock($fields)){
              $data = 'success';
            }else{
              $hasError = true;
              $data = "Error while updating record info. Please, try again later and contact support.";
            }
          }
          exit(json_encode(array("data"=>$data, "hasError" =>$hasError)));
        }elseif($action == 'delete_stock'){
          $id = isset($_POST["id"]) && $_POST["id"] ? intval($_POST["id"]) : '';
          $data = '';
          if($id == ''){
            $data .= "Stock is missing.";
            $hasError = true;
          }
          
          if(!$hasError){
            if($this->sentimentModel->removeStock($id)){
              $data = "success";
							$this->sentimentModel->changeUserStockPendingStatus($user->id, $user->tier);
            }else{
              $data = "Error while deleting record info. Please, try again later and contact support.";
            }
          }
          exit($data);
        }
      }
      
      
      $records = null;
      $order = null;
			$pages = null;
			$searchParams = new stdClass;
      $search = null;
      $legend = null;
      $mode = null;
      $exchange = null;
      $time = strtotime('now');
      $tickerIds = array();
      
      $type = isset($_GET["type"]) ? trim($_GET["type"]) : 'list';
      
      if ($type == "list")
      {
      	$this->sentimentModel->mode = (isset($_COOKIE['view_mode'])) ? $_COOKIE['view_mode'] : null;
				$sentimentField = ($this->sentimentModel->mode == 'MP') ? 'mp_sentiment' : 'sentiment';
        $url = '/dashboard/?page=PAGE';
        $searchParams->sorting = array(
        	'ticker' => array('value'=>'us.ticker', 'name'=>'TICKER', 'status' => true),
          'company' => array('value'=>'sd.company', 'name'=>'COMPANY', 'status' => true),
          'price' => array('value'=>'sd.price', 'name'=>'PRICE', 'status' => false),
          'price-change' => array('value'=>'sd.percent_change', 'name'=>'PRICE CHANGE(%)', 'status' => true),
          'sentiment' => array('value'=>'sd.'.$sentimentField , 'name'=>'SENTIMENT', 'status' => true),
          'buzz' => array('value'=>'sd.buzz', 'name'=>'BUZZ', 'status' => true)
        );
        $searchParams->limit = array(10, 25);
          			
  			if(isset($_COOKIE["options"]) && $_COOKIE["options"]) $search = json_decode(stripslashes($_COOKIE["options"]));
  			if(!$search){
  				$search = new stdClass;
  	      $search->sort = 'ticker';
  	      $search->order = 'asc';
  	      $search->limit = 10;
  				$search->page = 1;
  			}
  			
  			if(isset($_GET["sort"]) && $_GET["sort"]) $search->sort = trim($_GET["sort"]);
        if(isset($_GET["order"]) && $_GET["order"]) $search->order = trim($_GET["order"]);
        if(isset($_GET["limit"]) && $_GET["limit"]) $search->limit = intval($_GET["limit"]);
  			if(isset($_GET["page"]) && $_GET["page"]) $search->page = intval($_GET["page"]);
  			
  			$_GET["page"] = $search->page;
  			
  			setcookie("options", json_encode($search), 0, "/dashboard/");
  			
        if ($searchParams->limit && in_array($search->limit, $searchParams->limit))
        {
          $url .= '&limit='.$search->limit;
        }
        if ($search->sort && array_key_exists($search->sort, $searchParams->sorting))
        {
          $searchParams->sorting[$search->sort]['active'] = true;
          $order = $searchParams->sorting[$search->sort]['value'];
          if ($order) $order .= ' ' . ($search->order == 'asc' ? 'ASC' : 'DESC');
          $url .= '&sort='.$search->sort;
          $url .= '&order='.$search->order;
        }
        
        $params['USER_ID'] = $user->id;
        $where = " us.user_id='USER_ID'";
				$fields = "us.id, us.ticker, sd.exchange, sd.company, sd.".$sentimentField." AS sentiment, sd.price, sd.percent_change, sd.buzz, us.status";
        $recordsCount = $this->sentimentModel->getUserStocksCount($where, $params);
        $pages = $recordsCount ? $this->paginationModel->pages($url, $recordsCount, $search->limit) : null;
        $records = $recordsCount ? $this->sentimentModel->getUserStocks($where, $params, $this->paginationModel->limit, $order , $fields) : null;
      }
      else
      {
        $params['USER_ID'] = $user->id;
        $where = " us.user_id='USER_ID'";
        $userTickers = $this->sentimentModel->getUserTickers($where, $params, null, null, 'us.ticker');
        if ($userTickers)
        {
          $tickerIds = array();
          foreach($userTickers as $item)
          {
            $tickerIds[] = $item->ticker;
          }
          $mode = 'buzz';
          $exchange = 'ALL';
					if(isset($_COOKIE['view_mode'])) HeatmapModel::viewMode($_COOKIE['view_mode']);
          $tickers = $mode == 'sentiment' ? HeatmapModel::getSentimentTickers($exchange, $tickerIds) : HeatmapModel::getBuzzTickers($exchange, $tickerIds);
          $tickers = PriceModel::formatPrices($tickers,2,'PRICE','EXCHANGE');
          $records = $tickers ? json_encode(HeatmapModel::getTree("HEAT MAP: " . strtoupper($mode), $tickers, $mode)) : null;
          $legend = array('-1.0' => '', '-0.5' => '', 0 => '', '0.5' => '', '1.0' => '');
          foreach($legend as $i => $v) $legend[$i] = HeatmapModel::getColor($i);
          $time = HeatmapModel::getUnixTS();
          $this->sentimentModel->db();
        }
      }
      
      $hasError = !$this->userModel->checkUserProfile();
      if ($hasError) $status = 'You haven’t filled out your profile yet. Fill out your profile to help us serve you better. <a href="/account-settings/">Click here</a> to get started.';
      
      $templateParams = array(
        "has_error" => $hasError,
        "status" => $status,
        "header" => &$header,
        "user" => $user,
        "type" => $type,
        "records" => $records,
        "search" => $search,
        "search_params" => $searchParams,
        "pages" =>$pages,
        "legend" => $legend,
        "exchange" => $exchange,
        "mode" => $mode,
        "ticker_ids" =>  implode(',',$tickerIds),
        "time" => $time
      );
      return $this->displayTemplate("dashboard.tpl", $templateParams);
    }
  }

?>