<?php
	/**
	 * This file is part of the exporting module for Highcharts JS.
	 * www.highcharts.com/license
	 *
	 *
	 * Available POST variables:
	 *
	 * $filename  string   The desired filename without extension
	 * $type      string   The MIME type for export.
	 * $width     int      The pixel width of the exported raster image. The height is calculated.
	 * $svg       string   The SVG source code to convert.
	 */

	class HighchartsServer extends FJF_CMD
	{
		function __construct()
		{
		}

		function execute()
		{
			// set temp dir & temp file name
			$tmp = sys_get_temp_dir();
			$tempName = md5(rand());

			$type = (isset($_POST["type"])) ? trim($_POST["type"]) : "image/png";
			$svg = (isset($_POST["svg"])) ? trim(stripslashes($_POST["svg"])) : "";
			$filename = (isset($_POST["filename"])) ? trim($_POST["filename"]) : "chart";
			$width = (isset($_POST["width"])) ? intval($_POST["width"]) : 0;

			// check for malicious attack in SVG
			if (strpos($svg, "<!ENTITY") !== false)
			{
				exit("Execution is stopped, the posted SVG could contain code for a mailcious attack");
			}

			// allow no other than predefined types
			if ($type == "image/png")
			{
				$typeString = "-m image/png";
				$ext = "png";
			}
			elseif ($type == "image/jpeg")
			{
				$typeString = "-m image/jpeg";
				$ext = "jpg";
			}
			elseif ($type == "application/pdf")
			{
				$typeString = "-m application/pdf";
				$ext = "pdf";
			}
			elseif ($type == "image/svg+xml")
			{
				$ext = "svg";
			}

			$outfile = "$tmp/$tempName.$ext";

			if (isset($typeString))
			{
				// size
				$widthString = "";
				if ($width > 0)
				{
					$widthString = "-size $width" . "x";
				}

				// generate the temporary file
				if (!file_put_contents("$tmp/$tempName.svg", $svg))
				{
					exit("Couldn't create temporary file. Check that the directory permissions for the $tmp directory are set to 777.");
				}

				// do the conversion
				$cmd = "convert $widthString " . escapeshellarg("$tmp/$tempName.svg") . " " . escapeshellarg("$outfile");
				$output = shell_exec($cmd . " 2>&1");

				// catch error
				if (!is_file($outfile) || filesize($outfile) < 10)
				{
					exit("Error while converting SVG.");
					/*
					echo "<pre>$output</pre>";
					echo "Error while converting SVG. ";

					if (strpos($output, 'SVGConverter.error.while.rasterizing.file') !== false) {
						echo "
						<h4>Debug steps</h4>
						<ol>
						<li>Copy the SVG:<br/><textarea rows=5>" . htmlentities(str_replace('>', ">\n", $svg)) . "</textarea></li>
						<li>Go to <a href='http://validator.w3.org/#validate_by_input' target='_blank'>validator.w3.org/#validate_by_input</a></li>
						<li>Paste the SVG</li>
						<li>Click More Options and select SVG 1.1 for Use Doctype</li>
						<li>Click the Check button</li>
						</ol>";
					}
					*/
				}
				else
				{
					header("Content-Disposition: attachment; filename=\"$filename.$ext\"");
					header("Content-Type: $type");
					echo file_get_contents($outfile);
				}

				// delete it
				unlink("$tmp/$tempName.svg");
				unlink($outfile);
			}
			else if ($ext == "svg")
			{
				header("Content-Disposition: attachment; filename=\"$filename.$ext\"");
				header("Content-Type: $type");
				echo $svg;
			}
			else
			{
				exit("Invalid type.");
			}
		}
	}

?>