<?php
	/**
	* Front Controller for Web Application
	* Intercepts all requests and loads appropriate commands
	* from ./commands directory.
	* Each command contains webpage logic; commands are defined in the settings file (./settings.php)
	*
	*/

	include_once('settings_servers.php');

  error_reporting(FJF_SERVER == 'live' ? 0 : E_ALL);
  ini_set('display_errors', FJF_SERVER == 'live' ? '0' : '1');
	
	include_once('settings_https.php');

	//root of the application
	define('FJF_ROOT', $_SERVER["DOCUMENT_ROOT"] . '/fitch/');
	//define front web controller - path to this file
	define('WEB_CONTROLLER', '/fitch/index.php');

	//include local configuration
	if (file_exists(FJF_ROOT . '/settings.php'))
	{
		include_once(FJF_ROOT . '/settings.php');
	}
	else
	{
		echo '<h1>Web Framework Error: Failed to load application configuration file.</h1>';
		exit;
	}

	if (file_exists(FJF_ROOT . '/settings_db.php'))
	{
		include_once(FJF_ROOT . '/settings_db.php');
	}
	else
	{
		echo '<h1>Web Framework Error: Failed to load application DB configuration file.</h1>';
		exit;
	}

	if (!is_array($WEB_APPLICATION_CONFIG))
	{
		echo '<h1>Web Framework Error: Invalid Configuration File: WEB_APPLICATION_CONFIG is null.</h1>';
		exit;
	}

	if(isset($WEB_APPLICATION_CONFIG["date_default_timezone"])){
		date_default_timezone_set($WEB_APPLICATION_CONFIG["date_default_timezone"]);
	}

	/**
	* Include Web Application Base Class
	*/
	if (file_exists($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_BASE.php"))
	{
		include_once($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_BASE.php");
	}
	else
	{
		echo 'Failed to include Web Application Base class.';
		exit;
	}

	if (file_exists($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_BASE_RICH.php"))
	{
		include_once($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_BASE_RICH.php");
	}
	else
	{
		echo 'Failed to include Web Application Base Rich class.';
		exit;
	}

	/**
	* Include Smarty Template Engine
	*/
	if (file_exists($WEB_APPLICATION_CONFIG["smarty_path"] . "/Smarty.class.php"))
	{
		include_once($WEB_APPLICATION_CONFIG["smarty_path"] . "/Smarty.class.php");
	}
	else
	{
		echo 'Failed to include Smarty library.';
		exit;
	}

	/** Global Redirection (fixing missing trailing slashes in URLs) */
	FJF_BASE::global_redirect();

	/** request variables */
	$REQUEST_CONTAINER = $_REQUEST;
	if ($_SERVER['REQUEST_METHOD'] == 'GET') $REQUEST_CONTAINER = $_GET;
	if ($_SERVER['REQUEST_METHOD'] == 'POST')
	{
		if(is_array($_GET) && count($_GET) > 0) $_POST = array_merge ($_GET, $_POST);
		$REQUEST_CONTAINER = &$_POST;
	}

	/**
	* Instantiate & configure Smarty
	*/
	$smartyEngine = new Smarty();
	$smartyEngine->compile_check = true;
	$smartyEngine->debugging = false;
	$smartyEngine->template_dir = $WEB_APPLICATION_CONFIG["templates_path"] . "/";
	$smartyEngine->compile_dir = $WEB_APPLICATION_CONFIG["smarty_compile_path"];
	$smartyEngine->config_dir = $WEB_APPLICATION_CONFIG["smarty_config_path"];
	$smartyEngine->cache_dir = $WEB_APPLICATION_CONFIG["smarty_cache_path"];
	$smartyEngine->assign("web_controller", WEB_CONTROLLER);
	$smartyEngine->assign("request_cmd", $REQUEST_CONTAINER['cmd']);
	$smartyEngine->assign_by_ref("web_config", $WEB_APPLICATION_CONFIG);


	/**
	* Include Additional FJF functions for Smarty Template Engine
	*/
	if (file_exists($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_SMARTY_FUNCTIONS.php"))
	{
		include_once($WEB_APPLICATION_CONFIG["lib_path"] . "/FJF_SMARTY_FUNCTIONS.php");
	}
	else
	{
		echo 'Failed to include Additional FJF functions for Smarty.';
		exit;
	}

	/** request command */
	$REQUEST_CMD = &$REQUEST_CONTAINER['cmd'];

	FJF_BASE::log("Request: [" . $_SERVER['REQUEST_METHOD'] . "," . $REQUEST_CMD . "]", "controller");

	/** Load Command Definition Based on request */
	FJF_BASE::process_request($REQUEST_CMD);

	//for testing
	function test($var = null, $toExit = 1) { echo "<pre>"; print_r($var); echo "</pre>"; if ($toExit) { exit; } return true; }

?>
