<?php

	/**
	* The class contains various utility methods accessed statically
	*/

	/** include command definition file */
	if (file_exists($GLOBALS["WEB_APPLICATION_CONFIG"]["lib_path"] . "/FJF_CMD.php"))
	{
		include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["lib_path"] . "/FJF_CMD.php");
	} else
	{
		echo 'Failed to include required library: ' . $GLOBALS["WEB_APPLICATION_CONFIG"]["lib_path"] . "/FJF_CMD.php";
		exit;
	}

	class FJF_BASE
	{

		public static function get_db($db = null){
			$old = isset($GLOBALS["WEB_APPLICATION_CONFIG"]["db_host"]);
			if(!$old && !$GLOBALS["WEB_APPLICATION_CONFIG"]["db"]) return false;
			if($db === null || $db === false){
				if($old) return array(
					"host" => $GLOBALS["WEB_APPLICATION_CONFIG"]["db_host"], "port" => $GLOBALS["WEB_APPLICATION_CONFIG"]["db_port"],
					"user" => $GLOBALS["WEB_APPLICATION_CONFIG"]["db_user"], "password" => $GLOBALS["WEB_APPLICATION_CONFIG"]["db_password"],
					"name" => $GLOBALS["WEB_APPLICATION_CONFIG"]["db_name"]
				);
				$tmp = array_keys($GLOBALS["WEB_APPLICATION_CONFIG"]["db"]);
				$db = array_shift($tmp);
			}
			if(isset($GLOBALS["WEB_APPLICATION_CONFIG"]["db"][$db])) return $GLOBALS["WEB_APPLICATION_CONFIG"]["db"][$db];
			return FJF_BASE::get_db();
		}//get_db

		public static function set_db($db = null){
			if($GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]) FJF_BASE::db_close();
			$GLOBALS["WEB_APPLICATION_CONFIG"]["db_tmp"] = FJF_BASE::get_db($db);
		}//set_db

		public static function db_settings(){
			if(!isset($GLOBALS["WEB_APPLICATION_CONFIG"]["db_tmp"])) FJF_BASE::set_db();
			return $GLOBALS["WEB_APPLICATION_CONFIG"]["db_tmp"];
		}//db_settings

		public static function db_error()
		{
			return $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"];
		}

		/** debug logger */
		public static function log($str, $method = null, $sql = false)
		{
			if (!$GLOBALS["WEB_APPLICATION_CONFIG"]["debug_mode"]) return false;

			if($sql = false){
				$sqlFile = $GLOBALS["WEB_APPLICATION_CONFIG"]["app_data_path"].'/sql_log';
				if ($sqlFile != "" && file_exists($sqlFile) && is_writable($sqlFile))
				{
					$sqlFile = $sqlFile.'/log_'.date("Y_m_d");
					$data = $str . ";" . "\n";
					if ($fp = fopen($sqlFile, "a+"))
					{
						$res = fwrite($fp, $data);
						fclose($fp);
					}
				}
				return false;
			}

			if ($GLOBALS["WEB_APPLICATION_CONFIG"]["log_file"] != "" &&
				file_exists($GLOBALS["WEB_APPLICATION_CONFIG"]["log_file"]) &&
				is_writable($GLOBALS["WEB_APPLICATION_CONFIG"]["log_file"])
				)
			{
				$data = date("m/d/Y h:i:s") . " " . time() . "(" . $method . "): " . $str . "\n";
				$fp = fopen($GLOBALS["WEB_APPLICATION_CONFIG"]["log_file"], "a+");
				$res = fwrite($fp, $data);
				fclose($fp);
			}
		}

		/** processes request and loads appropriate command based on the definition */
		public static function process_request($cmdToLoad)
		{
			$cmdToLoad = strtolower(urldecode($cmdToLoad));
			if (trim($cmdToLoad) == '')
			{
				$cmdToLoad = $GLOBALS["WEB_APPLICATION_CONFIG"]["default_cmd"];
			}

			$file = $cmdToLoad . ".php";

			FJF_BASE::log("Processing '$cmdToLoad' ...", __METHOD__);
			if ($GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad] != null &&
				$GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad]["class"] != null &&
				file_exists($GLOBALS['WEB_APPLICATION_CONFIG']["commands_path"] . "/" . $file)
				)
			{
				FJF_BASE::log("Including file '" . $file . "' ...", __METHOD__);
				//include command file
				include_once($GLOBALS['WEB_APPLICATION_CONFIG']["commands_path"] . "/" . $file);
				//create new instance of the class
				if (class_exists($GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad]["class"]))
				{
					$tmpInstance = new $GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad]["class"];
					//execute the command
					$res = $tmpInstance->execute();
					unset($tmpInstance);
					return $res;
				} else
				{
					FJF_BASE::log("Class '" . $GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad]["class"] .  "' does not exist", __METHOD__);
					echo ("<h1>FJF Framework Error: Class '" . $GLOBALS['WEB_APPLICATION_CONFIG'][$cmdToLoad]["class"] .  "' does not exist.</h1>");
				}
			} else
			{
				FJF_BASE::log("Failed to process '$cmdToLoad': invalid configuration", __METHOD__);
				echo ("<h1>FJF Framework Error: Failed to process '$cmdToLoad': invalid configuration.</h1>");
			}
			return false;
		}

		/** redirects */
		public static function redirect($loc)
		{
			FJF_BASE::log("Redirecting to '$loc'", __METHOD__);
			//header("HTTP/1.1 301 Moved Permanently");
			if(session_id()) session_write_close();
			header("Location: $loc");
			exit;
		}

		function status($s = null)
		{
			if($s) $_SESSION["rstatus"] = $s;
			else
			{
				$s = isset($_SESSION["rstatus"]) ? $_SESSION["rstatus"] : "";
				unset($_SESSION["rstatus"]);
			}
			return $s;
		}//status

		/** global redirect */
		public static function global_redirect()
		{
			$toRedirect = false;
			$loc = $_SERVER["REQUEST_URI"];

			$exceptionCMD = array("widgets_gen", "script", "news_feed");

			if ($_SERVER["REQUEST_METHOD"] == "POST" || $_SERVER["REQUEST_METHOD"] == "PUT")
			{
				return true;
			}
			if (strpos($_SERVER["REQUEST_URI"], "?") !== false)
			{
				return true;
			}
			// if this is AJAX request, then no redirects
			elseif (preg_match("/\/ajax\//", $loc) || isset($_REQUEST["cmd"]) && substr($_REQUEST["cmd"], 0, 4) == "ajax")
			{
				return true;
			}
			// if this is ADMIN request, then no redirects
			elseif (preg_match("/^\/admin\//", $loc) || isset($_REQUEST["cmd"]) && substr($_REQUEST["cmd"], 0, 5) == "admin")
			{
				return true;
			}
			// if this is exception CMD
			elseif (isset($_REQUEST["cmd"]) && in_array($_REQUEST["cmd"], $exceptionCMD))
			{
				return true;
			}
			// if this is an image or css or js file (which was NOT found and is being redirected to Error404 page)
			elseif (preg_match("/(\.jpe?g$)|(\.gif$)|(\.png$)|(\.css$)|(\.js$)|(\.ttf$)|(\.pdf$)|(\.PDF$)|(\.eot$)/", $loc))
			{
				return true;
			}
			// no trailing slashes
			elseif (!preg_match("/\/$/", $loc))
			{
				$toRedirect = true;
			}
			// double trailing slashes (or more)
			elseif (preg_match("/\/{2,}$/", $loc))
			{
				$toRedirect = true;
				$loc = rtrim($loc, "/ ");
			}

			// checking if we need to redirect...
			if ($toRedirect)
			{
				FJF_BASE::log("GlobalRedirecting: FROM '" . $_SERVER["REQUEST_URI"] . "' TO '" . $loc . "/'", __METHOD__);
				header("HTTP/1.1 301 Moved Permanently");
				header("Location: " . $loc . "/");
				exit;
			}

			return true;
		}

		/**
		* Escapes string to avoid SQL Injection
		*/
		public static function db_escape($value)
		{
			if ($GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"] == false)
			{
				return null;
			}

			if (get_magic_quotes_gpc())
			{	$value = stripslashes($value);
			}
			$value = mysql_real_escape_string($value, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);

			return $value;
		}

		/**
		* Opens db connection: dbSetParams must be called first
		*/
		public static function db_open()
		{
			$db = FJF_BASE::db_settings();
			if(!$db){
				$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = "FJF Framework Error: Invalid DB Config.";
				FJF_BASE::log("Invalid DB Config.", __METHOD__);
				return false;
			}

			FJF_BASE::log("Params:[host=" . $db["host"] . ",port=" . $db["port"] . ",dbname=" . $db["name"] . ",user=" . $db["user"] . ",password=***]", __METHOD__);

			if ($db["port"] != "")
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"] = mysql_connect($db["host"] . ":" . $db["port"], $db["user"], $db["password"]);
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"] = mysql_connect($db["host"], $db["user"], $db["password"]);
			}

			if ($GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"] != false)
			{
				FJF_BASE::log("DB Connection Established", __METHOD__);
				if ($db["name"] != "")
				{	if (mysql_select_db($db["name"], $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]))
					{	FJF_BASE::log("Selected DB: " . $db["name"], __METHOD__);
					} else
					{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
						FJF_BASE::log("Failed to select DB: " . $db["name"], __METHOD__);
					}
				}
				return true;
			} else
			{
				$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("DB Connection Failed: " . $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
			}
			return false;
		}

		/**
		* Returns true if connected
		*/
		public static function db_is_connected()
		{
			return ($GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]) ? true : false;
		}

		/**
		* Updates database record
		*/
		public static function db_update($table,$data,$critiria,$debug = FALSE)
		{
			FJF_BASE::log("Params: [table:$table,data:" . print_r($data,true) . ",critiria:$critiria]", __METHOD__);
			$sql_temp = "";
			$sql = "UPDATE $table SET ";
			if (is_array($data))
			{	reset ($data);

				while (list($field, $value) = each($data))
				{
					//if ($value == null || strlen(trim($value)) == 0)
					if ($value === null)
					{
						$sql_temp .= $field . "=NULL,";

					} else
					{
						$sql_temp .= $field . "='" . FJF_BASE::db_escape($value) . "',";
					}
				}
				$sql .= substr($sql_temp,0,strlen($sql_temp)-1);

				if ($critiria != null)
				{
					$sql .= " WHERE " . $critiria;
				}
				FJF_BASE::log($sql, __METHOD__, true);

				if ($debug)
				{
					exit($sql);
				}

				$result = mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);
				if ($result)
				{	FJF_BASE::log("SUCCESS", __METHOD__);
					return true;
				} else
				{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
					FJF_BASE::log("FAILURE: ". $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
				}
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("FAILURE: Invalid Data Parameter", __METHOD__);
			}

			return null;
		}

		/**
		* Inserts record into database
		*/
		public static function db_insert($table,$data,$primary_key)
		{
			FJF_BASE::log("Params: [table:$table,data:" . print_r($data,true) . ",pk:$primary_key]", __METHOD__);
			$sql_temp1 = "";
			$sql_temp2 = "";
			$sql = "INSERT INTO $table (";
			if (is_array($data))
			{	reset ($data);

				while (list($field, $value) = each($data))
				{
					if ($field != $primary_key)
					{
						$sql_temp1 .= $field . ",";

						//if ($value == null || strlen(trim($value)) == 0)
				    if ($value === null)
						{
							$sql_temp2 .= "NULL,";

						} else
						{
							$sql_temp2 .= "'" . FJF_BASE::db_escape($value) . "',";
						}
					}
				}
				//$sql .= substr($sql_temp1,0,strlen($sql_temp1)-1) . ") VALUES (" . substr($sql_temp2,0,strlen($sql_temp2)-1) . ")";
			  $sql .= rtrim($sql_temp1, ", ") . ") VALUES (" . rtrim($sql_temp2, ", ") . ")";

				FJF_BASE::log($sql, __METHOD__, true);
				$result = mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);
				if ($result)
				{	$nextID = mysql_insert_id();
					FJF_BASE::log("SUCCESS. Returned value is: " . $nextID, __METHOD__);
					return $nextID;
				} else
				{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
					FJF_BASE::log("FAILURE: ". $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
				}
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("FAILURE: Invalid Data Parameter", __METHOD__);
			}

			return null;
		}

		public static function db_replace($table,$data)
		{
			FJF_BASE::log("Params: [table:$table,data:" . print_r($data,true) . "]", __METHOD__);
			$sql_temp1 = "";
			$sql_temp2 = "";
			$sql = "REPLACE INTO $table (";
			if (is_array($data))
			{	reset ($data);

				while (list($field, $value) = each($data))
				{
					$sql_temp1 .= $field . ",";
			    if ($value === null) $sql_temp2 .= "NULL,";
					else $sql_temp2 .= "'" . FJF_BASE::db_escape($value) . "',";
				}
			  $sql .= rtrim($sql_temp1, ", ") . ") VALUES (" . rtrim($sql_temp2, ", ") . ")";

				FJF_BASE::log($sql, __METHOD__, true);
				$result = mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);
				if ($result) return true;
				else
				{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
					FJF_BASE::log("FAILURE: ". $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
				}
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("FAILURE: Invalid Data Parameter", __METHOD__);
			}

			return null;
		}

		/**
		* Closes db connection
		*/
		public static function db_close()
		{
			/*
			if (mysql_close($GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]))
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"] = false;
				FJF_BASE::log("SUCCESS", __METHOD__);
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("FAILURE: " . $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
			}
			*/
		}
		/**
		* Returns single record for a table
		*/
		public static function db_get_one($table, $critiria, $sort = null)
		{
			FJF_BASE::log("Params: [table:$table,critiria:$critiria,sort:$sort]", __METHOD__);

			if ($critiria == "")
			{	FJF_BASE::log("Failed to get data: critiria is empty", __METHOD__);
				return null;
			}

			$sql = "SELECT * FROM $table WHERE $critiria";
			if ($sort != null)
			{	$sql .= " ORDER BY " . $sort;
			}
			FJF_BASE::log($sql, __METHOD__, true);
			$result = mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);
			if ($result)
			{	switch (mysql_num_rows ($result))
				{	case 0:
						FJF_BASE::log("No data was returned", __METHOD__);
						return 0;
					default:
						$record = mysql_fetch_object($result);
						return $record;
				}
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("Failed to get record: " . $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
			}
			return null;
		}
		/**
		* Returns many records based on sql query
		*/
		public static function db_get_many($sql)
		{
			FJF_BASE::log($sql, __METHOD__, true);

			$return = null;

			$result = mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]);
			if ($result)
			{	switch (mysql_num_rows ($result))
				{	case 0:
						FJF_BASE::log("No data was returned", __METHOD__);
						$return = 0;
					default:
						$records = array();
						for ($i=0;$i<mysql_num_rows ($result);$i++)
						{
							if (mysql_data_seek($result, $i) == 0) continue;
							$record = mysql_fetch_object($result);
							$key = isset($GLOBALS["WEB_APPLICATION_CONFIG"]["FJF_BASE_key"]) ? $record->$GLOBALS["WEB_APPLICATION_CONFIG"]["FJF_BASE_key"] : $i;
							$records[$key] = $record;
						}
						if (count($records) > 0)
						{	FJF_BASE::log("Got '" . count($records) . "' records", __METHOD__);
							$return = $records;
						}
				}
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("Failed to get records: " . $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
			}

			if(isset($GLOBALS["WEB_APPLICATION_CONFIG"]["FJF_BASE_key"])) unset($GLOBALS["WEB_APPLICATION_CONFIG"]["FJF_BASE_key"]);
			return $return;
		}
		/**
		* Executes sql query
		*/
		public static function db_execute($sql)
		{
			if (@mysql_query($sql, $GLOBALS["WEB_APPLICATION_CONFIG"]["db_connection"]))
			{	FJF_BASE::log("Successfully executed: $sql", __METHOD__, true);
				return true;
			} else
			{	$GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"] = mysql_error();
				FJF_BASE::log("Failed to execute: $sql. Error: " . $GLOBALS["WEB_APPLICATION_CONFIG"]["db_error"], __METHOD__);
			}
			return false;
		}

		/**
		* Does MD5 Encryption
		*/
		public static function encrypt($plain_text, $password, $iv_len = 16)
		{	$plain_text .= "\x13";
			$n = strlen($plain_text);
			if ($n % 16) $plain_text .= str_repeat("\0", 16 - ($n % 16));
			$i = 0;
			$enc_text = FJF_BASE::getRandChar($iv_len);
			$iv = substr($password ^ $enc_text, 0, 512);
			while ($i < $n) {
			   $block = substr($plain_text, $i, 16) ^ pack('H*', md5($iv));
			   $enc_text .= $block;
			   $iv = substr($block . $iv, 0, 512) ^ $password;
			   $i += 16;
			}
			$encStr = base64_encode($enc_text);
			FJF_BASE::log("Encrypting '$plain_text' into  #$%^&** ....", __METHOD__);
			return $encStr;
		}

		public static function decrypt($enc_text, $password, $iv_len = 16)
		{
			$enc_text = base64_decode($enc_text);
			$n = strlen($enc_text);
			$i = $iv_len;
			$plain_text = '';
			$iv = substr($password ^ substr($enc_text, 0, $iv_len), 0, 512);
			while ($i < $n) {
			   $block = substr($enc_text, $i, 16);
			   $plain_text .= $block ^ pack('H*', md5($iv));
			   $iv = substr($block . $iv, 0, 512) ^ $password;
			   $i += 16;
			}
			$res = preg_replace('/\\x13\\x00*$/', '', $plain_text);
			FJF_BASE::log("Decrypting #$%^&** into.... $res", __METHOD__);
			return $res;
		}

		public static function getRandChar($iv_len)
		{
		   $iv = '';
		   while ($iv_len-- > 0)
		   {
		       $iv .= chr(mt_rand() & 0xff);
		   }
		   return $iv;
		}

		public static function date_to_sql($st_date)
		{	if ($st_date != "")
			{	$ar_temp = explode("/", $st_date);
				if (is_array($ar_temp) == true)
				{	$st_return = date("Y-m-d",mktime(0,0,0,$ar_temp[0],$ar_temp[1],$ar_temp[2]));
					return $st_return;
				}
			}
			return false;
		}

		function date_from_sql($st_sqldate)
		{	if ($st_sqldate != "")
			{	$ar_dates = explode ("-", $st_sqldate);
				if (is_array($ar_dates) == true)
				{	$st_date = strftime("%m/%d/%Y", mktime(0,0,0,$ar_dates[1],$ar_dates[2],$ar_dates[0]));
					return $st_date;
				}
			}
			return false;
		}

		function date_is_expired($sql_date)
		{
			if (strtotime ($sql_date) < time() )
			{
				return true;
			}
		}

		function is_uploaded($st_filename)
		{   FJF_BASE::log($st_filename, __METHOD__);
			if ($st_filename != "")
			{	if (!$st_tmp_file = get_cfg_var("upload_tmp_dir"))
				{	//$st_tmp_file = dirname(tempnam("", ""));
			    }

			    $st_tmp_file = '/tmp';

			    $st_tmp_file .= "/" . basename($st_filename);

			    return (ereg_replace("/+", "/", $st_tmp_file) == $st_filename);
			}
			return false;
		}

		function is_extension_allowed($file)
		{
			FJF_BASE::log(print_r(func_get_args(), true), __METHOD__);
			return true;
		}

		function reverse_string($str)
		{	$st_return = null;
			if ($str != "")
			{
				for ($in_A=strlen($str); $in_A>=1; $in_A--)
				{	$st_return .= $str[$in_A-1];
				}
			}
			return $st_return;
		}

		function get_file_extension($file)
		{
			if ($file != '')
			{
				if (ereg("\.", $file))
				{	$st_extension = strtok(FJF_BASE::reverse_string($file), ".");
					return FJF_BASE::reverse_string($st_extension);
				}
			}
		}

		function upload_file($file, $dir_dest, &$message, $file_path_name)
		{	FJF_BASE::log(print_r(func_get_args(), true), __METHOD__);

			if (FJF_BASE::is_extension_allowed($file))
			{	if (FJF_BASE::is_uploaded($file))
				{	$newFileNameReplaced = @preg_replace("/\s+|:|\|\//","-", $file_path_name);

					$st_new_file_path = $dir_dest . "/" . $newFileNameReplaced;
					if (@copy($file, $st_new_file_path))
					{	@unlink($file);
						FJF_BASE::log("Successfully Uploaded File", __METHOD__);
						return true;
					} else $message = "Failed to upload file: Can't copy file to " . $st_new_file_path;
				} else
				{	$message = "Failed to upload file '$file_path_name'. Possible reasons: <br/>\n";
					$message .= "1. Permission Denied. Make sure PHP has a write access to the file output directory.<br/>\n";
					$message .= "2. File size is larger than the maximum size allowed (" . ini_get("upload_max_filesize") . ").<br/>\n";
				}
			} else
			{	$message = "Failed to upload file: this is a forbidden file type";
			}
			FJF_BASE::log("Failed to upload file '$file'. Error: " . $message, __METHOD__);
			return false;
		}

		public static function replace_params($str, $params)
		{
			if(is_array($params) && !empty($params))
			{
				foreach($params as $search => $replace) $str = str_replace($search, FJF_BASE::db_escape($replace), $str);
			}
			return $str;
		}

		function serialize($v)
		{
			$r = '';
			if(is_null($v)) return 'N;';
			elseif(is_bool($v)) return 'b:' . intval($v) . ';';
			elseif(is_int($v)) return 'i:' . $v . ';';
			elseif(is_float($v)) return 'd:' . $v . ';';
			elseif(is_string($v)) return 's:' . strlen(utf8_decode($v)) . ':"' . $v . '";';
			elseif(is_array($v))
			{
				$r .= 'a:' . count($v) . ':{';
				foreach($v as $k => $i) $r .= FJF_BASE::serialize($k) . FJF_BASE::serialize($i);
				$r .= '}';
			}
			elseif(is_object($v))
			{
				$n = get_class($v);
				$r .= 'O:' . strlen($n) . ':"' . $n . '":' . count(get_object_vars($v)) . ':{';
				foreach($v as $k => $i) $r .= FJF_BASE::serialize($k) . FJF_BASE::serialize($i);
				$r .= '}';
			}
			return $r;
		}//serialize

		function parseXML($xml)
		{
			$xml = new SimpleXMLElement($xml);

			if($xml === false) return false;

			function parseXML($xml)
			{
				if(strlen(trim($xml.""))) $xml = $xml."";
				if(is_object($xml) && $xml->children())
				{
					$result = null;
					foreach($xml as $key => $value)
					{
						if(!isset($result[$key])) $result[$key] = parseXML($value);
						else
						{
							if(is_string($result[$key])) $result[$key] = array($result[$key]);
							$result[$key][] = parseXML($value);
						}
					}
					return $result;
				}
				return trim($xml."");
			}//parseXML

			return parseXML($xml);
		}//parseXML
	}

?>