<?php

  /**
  * Blog Model
  */

  class BlogModel extends FJF_BASE_RICH
  {
    var $time = null;
    
    function __construct()
    {
      $this->time = date('Y-m-d H:i:s');
    }

    function getBlogs($where = null, $params = null, $limit = null, $order = null, $fields = null)
    {
      $sql = "SELECT ". ($fields ? $fields :"b.id, b.name, b.url_title, b.tracking_code, b.photo AS blog_photo,u.first_name, u.last_name");
      $sql .= " FROM blogs b";
      $sql .= " LEFT JOIN users u ON (b.author_id = u.id)";
      $sql .= ($where) ? " WHERE ".$where : '';
      if ($order) $sql .= " ORDER BY ".$order;
      if ($limit) $sql .= " LIMIT ".$limit;
      $blogs = $this->selectRecords($sql, $params);
      return $blogs;
    } // /getBlogs()
    
    function getBlogsCategories($where = null, $params = null, $limit = null, $order = null, $fields = null)
    {
      $sql = "SELECT ". ($fields ? $fields :"bc.id, bc.title, bc.url_title");
      $sql .= " FROM blogs_categories bc";
      $sql .= " LEFT JOIN blogs b ON b.id = bc.blog_id";
      $sql .= ($where) ? " WHERE ".$where : '';
      $sql .= " ORDER BY ".($order ? $order : "bc.title");
      if ($limit) $sql .= " LIMIT ".$limit;
      $records = $this->selectRecords($sql, $params);
      return $records;
    } // /getBlogs()
    
    function getBlogEntries($where = null, $params = null, $limit = null, $order = null, $fields = null, $group = null)
    {
      $sql = "SELECT ". ($fields ? $fields :"be.id, be.title, be.subtitle, be.url_title, be.summary, be.image, be.datetime, be.categories_ids, CONCAT(u.first_name, ' ', u.last_name) AS username");
      $sql .= " FROM blogs_entries be";
      $sql .= " LEFT JOIN users u ON (be.author_id = u.id)";
      $sql .= " LEFT JOIN blogs b ON (be.blog_id = b.id)";
      $sql .= " LEFT JOIN blogs_categories bc ON (be.blog_category_id = bc.id)";
      $sql .= " WHERE be.status = 1 AND be.published = 1";
      $sql .= " AND be.datetime < '".$this->time."'";
      $sql .= ($where) ? " AND ".$where : '';
      if ($group) $sql .= " GROUP BY " . $group;
      $sql .= " ORDER BY ".($order ? $order : "be.datetime DESC");
      if ($limit) $sql .= " LIMIT " . $limit;
      //test($sql);
      $blogEntries =  $this->selectRecords($sql, $params);
      return $blogEntries;
    } // /getBlogEntries()
    
    function getBlogEntriesCount($where = null, $params = null)
    {
      $result =  $this->getBlogEntries($where, $params, null, null, 'COUNT(be.id) AS count');
      return ($result) ? $result[0]->count : 0;
    } // /getBlogEntriesCount()
    
    function getBlogEntriesRecord($where = null, $params = null, $fields = null, $order = null)
    {
      $fields = ($fields) ? $fields : "be.id, be.title, be.categories_ids, be.subtitle, be.description, be.datetime, u.first_name, u.last_name, be.meta_title, be.meta_description, be.meta_keywords, be.extra_header_tag, CONCAT(u.first_name, ' ', u.last_name) AS username";
      $blogEntry = $this->getBlogEntries($where, $params, 1, $order, $fields);
      return ($blogEntry) ? $blogEntry[0] : null;
    } // /getBlogEntriesRecord()
    
    
    function getBlogEntriesCategories($where = null, $params = null, $limit = null, $order = null, $fields = null)
    {
      $sql = "SELECT ". ($fields ? $fields :"bec.id, bec.title, bec.url_title");
      $sql .= " FROM blogs_entries_categories bec";
      $sql .= " LEFT JOIN blogs b ON b.id = bec.blog_id";
      $sql .= " WHERE 1";
      $sql .= ($where) ? " AND ".$where : '';
      $sql .= " ORDER BY ".($order ? $order : "bec.title");
      if ($limit) $sql .= " LIMIT " . $limit;
      $records =  $this->selectRecords($sql, $params);
      return $records;
    }
    
    function getBlogEntriesCategory($where = null, $params = null, $fields = null, $order = null)
    {
      $fields = ($fields) ? $fields : 'bec.id, bec.title, bec.url_title';
      $record = $this->getBlogEntriesCategories($where, $params, 1, $order, $fields);
      return ($record) ? $record[0] : null;
    } // /getBlogEntriesCategory()
    
    function getArchiveDate()
    {
    	$result = array();
    	$records = $this->getBlogEntries(null, null, 12, "be.datetime DESC", "be.datetime, DATE_FORMAT(be.datetime,'%M %Y') AS title", "title");
      if($records){
				foreach($records as $item){
					$result[] = (object) array(
						"title" => $item->title,
						"url_title" => date('Y-n',strtotime($item->datetime))
					);
				}
			}
			/*
			$result[] = (object) array(
				"title" => 'OLDER',
				"url_title" => 'older'
			);
			*/
      return $result;
    }


  } // /BlogModel

?>