<?php

class HeatmapModel{
	
	private static $dataTableName = "HeatMapData";
	private static $sentimentFieldName = "AVERAGE_SENTIMENT";
	
	static $exchanges = array(
		"NASDAQ",
		"NEW YORK STOCK EXCHANGE, INC.",
		"NYSE AMEX EQUITIES",
		"NYSE ARCA",
		"OTC BULLETIN BOARD",
		"OTC PINK MARKETPLACE",
		"OTHER OTC/NBB",
		"Various Exchanges"
	);
		
	static $replace = array(
		"NEW YORK STOCK EXCHANGE, INC." => "NYSE",
		"OTC BULLETIN BOARD" => "OTC BB"
	);
	
	static function viewMode($mode){
		HeatmapModel::$dataTableName = $mode == 'MP' ? "HeatMapData_MP" : "HeatMapData";
		HeatmapModel::$sentimentFieldName = $mode == 'MP' ? "SIGNAL_VALUE" : "AVERAGE_SENTIMENT";
	}//viewMode
  
	static function getColor($a){
		$colors = array(
			'zero' => array(68, 68, 68),
			'max' => array(4, 190, 0),
			'min' => array(255, 0, 0)
		);
		
		if($a == 0) $color = $colors['zero'];
		else{
			$color = array();
			foreach($colors['zero'] as $i => $v) $color[$i] = round($v + ($colors[$a < 0 ? 'min' : 'max'][$i] - $v) * abs($a));
		}
		return 'rgb(' . implode(',', $color) . ')';
	}//getColor

	static function getTS($unix = false){
		static $TS;
		static $unixTS;
		if(!isset($TS)){
			FJF_BASE_RICH::db('sentiment');
			if($result = FJF_BASE_RICH::selectRecords("SELECT TS, UNIX_TIMESTAMP(TS) AS UNIX_TS FROM " . HeatmapModel::$dataTableName . " ORDER BY TS DESC LIMIT 1", null, true)){
				$TS = $result->TS;
        $unixTS = $result->UNIX_TS;
			}
		}
		return $unix ? $unixTS : $TS;
	}//getTS
	
	static function getUnixTS(){
    return HeatmapModel::getTS(true);
	}//getUnixTS
	
	static function getExchanges(){
		$records = HeatmapModel::$exchanges;
		foreach($records as $i => $v){
			if(array_key_exists($v, HeatmapModel::$replace)) $records[$i] = HeatmapModel::$replace[$v];
		}
		return $records;
		/*
		if($TS = HeatmapModel::getTS()){
			$params = array();
			$sql = "";
			$sql .= "SELECT DISTINCT EXCHANGE_NAME NAME";
			$sql .= " FROM " . HeatmapModel::$dataTableName;
			//$sql .= " WHERE TS='" . $TS . "'";
			$sql .= " ORDER BY NAME";
			$GLOBALS["WEB_APPLICATION_CONFIG"]["FJF_BASE_key"] = "NAME";
			if($records = FJF_BASE_RICH::selectRecords($sql)){
				$records = array_keys($records);
				foreach($records as $i => $v){
					if(array_key_exists($v, HeatmapModel::$replace)) $records[$i] = HeatmapModel::$replace[$v];
				}
				return $records;
			}
		}
		return null;
		 */
	}//getExchanges
	
	static function prepareExchange($exchange){
		if($exchange == "ALL") return;
		$e = array_search($exchange, HeatmapModel::$replace);
		return $e !== false ? $e : $exchange;
	}//prepareExchange
	
	static function getNearestSquareNumber($num){
		return pow(ceil(sqrt($num)), 2);
	}//getNearestSquareNumber
	
	static function getSentimentTickers($exchange, $tickers = null){
		$exchange = HeatmapModel::prepareExchange($exchange);
		
		if($TS = HeatmapModel::getTS()){
			$params = null;
      if ($exchange) $params['EXCHANGE_VAL'] = $exchange;
      if ($tickers && is_array($tickers)) $params['TICKERS_IDS'] = implode("','", $tickers);
			
			$sql = "";
			$sql .= "SELECT SECTOR, COUNT(DISTINCT TICKER) TOTAL";
			$sql .= " FROM " . HeatmapModel::$dataTableName;
			$sql .= " WHERE TS='" . $TS . "'";
			if($exchange) $sql .= " AND EXCHANGE_NAME='EXCHANGE_VAL'";
      if(isset($params['TICKERS_IDS'])) $sql .= " AND TICKER IN('".$params['TICKERS_IDS']."')";
			$sql .= " GROUP BY SECTOR";
			$sql .= " ORDER BY SECTOR";
			if($sectors = FJF_BASE_RICH::selectRecords($sql, $params)){
				
				$limit = 0;
				foreach($sectors as $sector) $limit = max($limit, $sector->TOTAL);
				$limit = HeatmapModel::getNearestSquareNumber($limit);
				
				$sql = array();
				foreach($sectors as $sector){
					$s = "SELECT TICKER";
					$s .= ", COMPANY_NAME COMPANY";
					$s .= ", EXCHANGE_NAME EXCHANGE";
					$s .= ", SECTOR";
					$s .= ", " . HeatmapModel::$sentimentFieldName . " SENTIMENT";
					$s .= ", BUZZ";
					$s .= ", LAST_PRICE PRICE";
					$s .= ", PERCENT_CHANGE";
					$s .= ", LAST_PRICE - (LAST_PRICE * 100) / (100 + PERCENT_CHANGE) `CHANGE`";
					$s .= " FROM " . HeatmapModel::$dataTableName;
					$s .= " WHERE TS='" . $TS . "'";
					if($exchange) $s .= " AND EXCHANGE_NAME='EXCHANGE_VAL'";
          if(isset($params['TICKERS_IDS'])) $sql .= " AND TICKER IN('".$params['TICKERS_IDS']."')";
					$s .= " AND SECTOR='" . $sector->SECTOR . "'";
					$s .= " LIMIT " . $limit;
					$sql[] = $s;
				}
				
				return FJF_BASE_RICH::selectRecords("(" . implode(") UNION (", $sql) . ") ORDER BY SECTOR, TICKER", $params);
			}
		}
		
		return null;
	}//getSentimentTickers
	
	static function getBuzzTickers($exchange, $tickers = null){
		$exchange = HeatmapModel::prepareExchange($exchange);
		
		if($TS = HeatmapModel::getTS()){
			$params = null;
      if ($exchange) $params['EXCHANGE_VAL'] = $exchange;
			if ($tickers && is_array($tickers)) $params['TICKERS_IDS'] = implode("','", $tickers);
			
			//C = P - (P * 100) / (100 + PC)
			
			$sql = "SELECT TICKER";
			$sql .= ", COMPANY_NAME COMPANY";
			$sql .= ", EXCHANGE_NAME EXCHANGE";
			$sql .= ", SECTOR";
			$sql .= ", " . HeatmapModel::$sentimentFieldName . " SENTIMENT";
			$sql .= ", BUZZ";
			$sql .= ", LAST_PRICE PRICE";
			$sql .= ", PERCENT_CHANGE";
			$sql .= ", LAST_PRICE - (LAST_PRICE * 100) / (100 + PERCENT_CHANGE) `CHANGE`";
			$sql .= " FROM " . HeatmapModel::$dataTableName;
			$sql .= " WHERE TS='" . $TS . "'";
			if(isset($params['EXCHANGE_VAL'])) $sql .= " AND EXCHANGE_NAME='EXCHANGE_VAL'";
      if(isset($params['TICKERS_IDS'])) $sql .= " AND TICKER IN('".$params['TICKERS_IDS']."')";
			//$sql .= " ORDER BY SECTOR, TICKER";
			$sql .= " LIMIT 200";
			return FJF_BASE_RICH::selectRecords($sql, $params);
		}
		
		return null;
	}//getBuzzTickers
	
	static function getTree($title = "Root", $tickers, $mode = null){
		$tree = array(
			'name' => $title,
			'id' => 'root',
			'data' => array(
				'$color' => '#000',
				'type' => 'group'
			),
			'children' => array()
		);
		
		$sectors = array();
		foreach($tickers as $i => $record){
			if(!array_key_exists($record->SECTOR, $sectors)) $sectors[$record->SECTOR] = array(
				'name' => $record->SECTOR,
				'id' => $record->SECTOR . '-sector-' . $i,
				'data' => array(
					'$color' => '#000',
					'$area' => $mode == 'sentiment' ? 1 : 0,
					'type' => 'group'
				),
				'children' => array()
			);
			//$record->SENTIMENT = rand(-100, 100)/100;
			//$record->BUZZ = rand(1, 200);
			$area = $mode == 'sentiment' ? 1 : $record->BUZZ + 0.1;
			$sectors[$record->SECTOR]['children'][] = array(
				'name' => '<span>'.$record->TICKER.'</span>',
				'id' => $record->TICKER . '-ticker-' . $i,
				'data' => array(
					'$color' => HeatmapModel::getColor($record->SENTIMENT),
					'$area' => $area,
					'type' => 'stock',
					'stock' => $record
				),
				'children' => array()
			);
			if($mode != 'sentiment') $sectors[$record->SECTOR]['data']['$area'] += $area;
		}
		
		if($mode == 'sentiment' && $sectors){
			$counter = 0;
			
			$totalSectors = count($sectors);
			for($m = 0, $n = HeatmapModel::getNearestSquareNumber($totalSectors) - $totalSectors; $m < $n; $m++){
				$sectors[] = array(
					'name' => 'No Current Sentiment Available',
					'id' => 'zzzzzzzzz-dummy-' . $counter++,
					'data' => array(
						'$color' => '#000',
						'$area' => 1,
						'type' => 'dummy-group'
					),
					'children' => array()
				);
			}
			
			$max = 0;
			foreach($sectors as $sector) $max = max($max, count($sector['children']));
			
			if($max){
				$max = HeatmapModel::getNearestSquareNumber($max);
				foreach($sectors as $i => $sector){
					$sectors[$i]['data']['$area'] = 1;
					for($m = 0, $n = $max - count($sectors[$i]['children']); $m < $n; $m++){
						$sectors[$i]['children'][] = array(
							'name' => '',
							'id' => 'zzzzzzzzz-dummy-' . $counter++,
							'data' => array(
								'$color' => '#000',
								'$area' => 1,
								'type' => 'dummy'
							)
						);
					}
				}
			}
		}
		
		$tree['children'] = array_values($sectors);
		unset($sectors, $tickers);
		
		return $tree;
	}//getTree
	
}

?>