<?php
	
	include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/file_model.php");

	class PhotoModel extends FileModel
	{
		private $thumbs = array();
		
		function __construct()
		{
			$this->messages["error_resize_image"] = "Error: Cannot resize image.";
		}
    
    
    function resizeImage($imageFilePath, $newImageFilePath, $params = array()){
      $newWidth = $params["width"];
      $newHeight = $params["height"];

      $imageType = pathinfo($imageFilePath, PATHINFO_EXTENSION);

      switch($imageType){
        case "jpg":
        case "jpeg":
          $returnVal = ($image = imagecreatefromjpeg($imageFilePath));
          break;
        case "gif":
          $returnVal = ($image = imagecreatefromgif($imageFilePath));
          break;
        default:
          $returnVal = ($image = imagecreatefrompng($imageFilePath));
      }
      
      $imageWidth = imagesx($image);
      $imageHeight = imagesy($image);

      if($imageWidth == $newWidth && $imageHeight == $newHeight){
        imagedestroy($image);
        return copy($imageFilePath, $newImageFilePath);
      }

      $newImage = imagecreatetruecolor($newWidth, $newHeight);
      switch($imageType){
        case "gif":
          imagecolortransparent($newImage, imagecolorallocate($newImage, 0, 0, 0));
          break;
        case "jpeg":
        case "jpg":
          imagefill($newImage, 0, 0, imagecolorallocate($newImage, 0xFF, 0xFF, 0xFF));
          break;
        default:
          imagealphablending($newImage, false);
          imagefill($newImage, 0, 0, imagecolorallocatealpha($newImage, 255, 255, 255, 127));
          imagealphablending($newImage, true);
          break;
      }
      
      if($imageWidth <= $newWidth && $imageHeight <= $newHeight){
        $dw = $imageWidth;
        $dh = $imageHeight;
      }else{
        $dr = $newWidth / $newHeight;
        $sr = $imageWidth / $imageHeight;
        $dw = $dr <= $sr ? $newWidth : round($newHeight * $sr);
        $dh = $dr >= $sr ? $newHeight : round($newWidth / $sr);
      }
      
      imagecopyresampled(
        $newImage,
        $image,
        round(($newWidth - $dw) / 2), round(($newHeight - $dh) / 2),
        0, 0,
        $dw, $dh,
        $imageWidth, $imageHeight
      );
      imagedestroy($image);
      
      switch($imageType){
        case "gif":
        case "jpeg":
        case "jpg":
          break;
        default:
          imagealphablending($newImage, false);
          imagesavealpha($newImage, true);
          break;
      }

      switch($imageType){
        case "jpg":
        case "jpeg":
          $returnVal = ($full = imagejpeg($newImage, $newImageFilePath, 90)) && $returnVal;
          break;
        case "gif":
          $returnVal = ($full = imagegif($newImage, $newImageFilePath)) && $returnVal;
          break;
        default:
          $returnVal = ($full = imagepng($newImage, $newImageFilePath)) && $returnVal;
      }

      chmod($newImageFilePath, 0777);
      imagedestroy($newImage);

      return $returnVal;
    }//resizeImage

		function defineThumb($f, $w, $h = false, $fillWithBg = true)
		{
			if($fillWithBg)
			{
				$newHeight = $h ? $h : $w;
			}
			else
			{
				$newHeight = 0;
			}
			$this->thumbs[$f] = array("folder" => $f, "width" => $w, "height" => $newHeight);
		}//defineThumb
		
		function upload($folder, $newFileName = null, $fieldName = "image", $defaultThumbs = false, $multipleItemKey = null)
		{
			$error = false;
			$status = "";

			$ALLOWED_EXT = array("gif", "jpg", "jpeg", "png");
			$filesPath = preg_replace("/\/+$/","",$_SERVER["DOCUMENT_ROOT"]) . "/images/" . $folder . "/";  // with trailing slash!
			
			if($defaultThumbs)
			{
				$this->defineThumb("s", 100);
				$this->defineThumb("m", 200);
			}
			$file = $this->_upload($filesPath . "orig/", $newFileName, $fieldName, $ALLOWED_EXT, 10485760, $multipleItemKey);
			if($file["error"]) $error = $file["error"];
			elseif(!empty($this->thumbs))
			{
				foreach($this->thumbs as $thumb)
				{
					if(!$this->resizeImage($filesPath . "orig/" . $file["file"], $filesPath . $thumb['folder'] . "/"  . $file["file"],  $thumb, $thumb))
					{
						$error = "error_resize_image";
						break;
					}
				}//end foreach
			}

			if($error) $status = $this->message($error);

			return array("error" => $error, "status" => $status, "file" => $file["file"]);
		} // /upload()
		
		function multipleUpload($folder, $newFileName = null, $fieldName = "images", $defaultThumbs = false, $imagesCount = 0){
		  $images = array();
		  if($imagesCount > 0){
		    for($i=0;$i<$imagesCount;$i++) $images[] = $this->upload($folder, $newFileName . $i, $fieldName, $defaultThumbs, $i);
		  }
      return $images;
		}
		
	} // end class PhotoModel
?>