<?php

	class SessionModel extends FJF_BASE_RICH
	{
		public $uid = 0;

		function __construct()
		{
			FJF_BASE::log(__METHOD__);
			if(session_id() == ''){
				session_start();
			}

			$refererArray = array("404error", "login", "register", "logout", "forgot-password");
			$isApiCommand = @preg_match("/\/api\//i", $_SERVER["REQUEST_URI"]) ? true : false;
			$isAjaxCommand = @preg_match("/\/ajax\//i", $_SERVER["REQUEST_URI"]) ? true : false;

			if(!(in_array(trim(strtolower($GLOBALS["REQUEST_CONTAINER"]["cmd"])), $refererArray) || $isApiCommand || $isAjaxCommand))
			{
				$_SESSION["app_next_cmd"] = trim($_SERVER['REQUEST_URI'], " /") ? urlencode($_SERVER['REQUEST_URI']) : "";
			}

			$this->uid = isset($_SESSION["userid"]) ? FJF_BASE::decrypt($_SESSION["userid"], "userid") : 0;
      if (!$this->uid)
      {
  			$userInfo = null;
        if(isset($_COOKIE["userid"]) && $_COOKIE["userid"])//USER HAS COOKIES
  			{
  				$userID = FJF_BASE::decrypt($_COOKIE["userid"], "userid");
  				$userInfo = $this->getRecordBy("users", $userID);
  			}
        if($userInfo) $this->setUserSession($userInfo);
      }
			$this->checkTierLevel();
			return true;
		}//SessionModel()

		////
		// checks for valid user in the database and returns userInfo on success
		//
		function checkUserExists($username, $password)
		{
			FJF_BASE::log(__METHOD__);
			return $this->getRecord("users", "password='PASSWORD' AND " . (preg_match("/^.+\@.+\..+$/", $username) ? "email" : "username") . "='USERNAME'", array("PASSWORD" => $password, "USERNAME" => $username));
		}//checkUserExists()


		function isAdmin()
		{
			return $this->isLoggedUser() ? $_SESSION["user"]->is_admin : 0;
		} // end isAdmin()

		function isLoggedUser()
		{
			return $this->uid > 0;
		} // end isLoggedUser()


		////
		// Sets user session params.
		//
		function setUserSession($userInfo, $setCookie = false, $disableRedirect = false)
		{
			FJF_BASE::log(__METHOD__);
			if(!$userInfo) return false;

			$app_next_cmd = isset($_SESSION["app_next_cmd"]) ? $_SESSION["app_next_cmd"] : null;
			//$this->unsetUserSession();
			
			$time = time();
			$userInfo->user_name = $userInfo->first_name . " " . $userInfo->last_name;
			$userInfo->password = null;
			$photo = $this->getFrom("photos", "type='users' AND item_id=" . $userInfo->id . " ORDER BY is_default DESC, id LIMIT 1", null, true);
			$userInfo->user_pic = $photo ? $photo->file : "";
			
			$this->uid = $userInfo->id;
			$_SESSION["userid"] = FJF_BASE::encrypt($userInfo->id, "userid");
			$_SESSION["user"] = $userInfo;
			if($userInfo->is_admin == 1)
			{
				$permissions = $this->getRecordBy("admin_permissions", $userInfo->id, "user_id");
				$_SESSION["user"]->permissions = $permissions ? explode(",", $permissions->permissions) : array();
			}

			$this->saveRecord("users", array("id" => $userInfo->id, "date_login" => date("Y-m-d H:i:s", $time)));

			if($setCookie)
			{
				$timeInSeconds = $time + 604800; // 7 days (7 * 24 * 60 * 60 seconds)
				setcookie("userid", $_SESSION["userid"], $timeInSeconds, "/");
			}
			
			if(!$disableRedirect){
				$redirect = $_SERVER["REQUEST_URI"];
				if($app_next_cmd)
				{
					$refererArray = array("error", "login", "register", "logout", "forgot-password", "");
					if(!in_array(trim(strtolower($app_next_cmd), " /"), $refererArray)) $redirect = urldecode($app_next_cmd);
				}
				$this->doPost();
				FJF_BASE::redirect($redirect);
			}
      return true;
		}//setUserSession()



		////
		// Deletes user session vars. used for logout action
		//
		function unsetUserSession()
		{
			FJF_BASE::log(__METHOD__);
			if(isset($_COOKIE["userid"])) setcookie("userid", "", time() - 3600, "/");
			//session_unset();
			$_SESSION["userid"] = null;
			$_SESSION["user"] = null;
			$this->uid = null;
			return true;
		} // unsetUserSession()
		
		function login($user = null, $setCookie = false, $skipRedirect = false)
    {
      if(!$user)
      {  
        $email = isset($_POST["email"]) ? trim($_POST["email"]) : "";
        $password = isset($_POST["password"]) ? trim($_POST["password"]) : "";
        $setCookie = isset($_POST["remember"]) ? intval($_POST["remember"]) : false;
        $user = $this->getRecord("users", "email='EMAIL' AND password='PASSWORD'", array("EMAIL" => $email, "PASSWORD" => $password));
      }
      if(!$user) return $this->error("Invalid login information.");
      elseif(!$user->active) return $this->error("This account is not active.");
      elseif($user->banned) return $this->error("This account is banned.");
      elseif($user->approved) return $this->error("This account is not approved.");
			elseif($user->cancelled) return $this->error("Please sign up again.");
      else{
        if ($this->setUserSession($user, $setCookie, true))
        {
        	if(!$skipRedirect){
	          $redirect = ($this->checkUserStocks()) ? '/dashboard/' : '/';
	          FJF_BASE::redirect($redirect);
					}
        }
      }
      return true;
    }//login

		function logout($redirect = "/")
		{
			FJF_BASE::log(__METHOD__);
			if($this->isLoggedUser())
			{
				$this->unsetUserSession();
			}
			FJF_BASE::redirect($redirect);
		}//logout

		////
		// Checks if username already exists in the DB.
		// Returns user ID if the username is already in use otherwise returns NULL
		//
		function checkUsernameExists($id)
		{
			FJF_BASE::log(__METHOD__);
			$result = $id ? $this->getRecordBy("users", $id, "username") : null;
			return $result ? $result->id : 0;
		}//checkUsernameExists()



		////
		// Checks if email already exists in the DB.
		// Returns user ID if the email is already in use otherwise returns NULL
		//
		function checkEmailExists($id)
		{
			FJF_BASE::log(__METHOD__);
			$result = $id ? $this->getRecordBy("users", $id, "email") : null;
			return $result ? $result->id : 0;
		}//checkEmailExists()
		
		function loginRedirect($url = null){
      if(!$url) $url = $this->getRedirectUrl();
      if($this->isLoggedUser()) FJF_BASE::redirect($url ? $url : "/");
    }//loginRedirect
    
    function notLoginRedirect(){
      if(!$this->isLoggedUser()){
	      $isAjax = !empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';
	      if ($isAjax) exit;
	      FJF_BASE::redirect("/register/");
      }
    }//notLoginRedirect
    
    function getUser()
    {
      $this->notLoginRedirect();
      return  (isset($_SESSION["user"])) ? $_SESSION["user"] : null;
    }

    function getRedirectUrl()
    {
      $redirect = "";
      if(isset($_SESSION["redirect"]) && $_SESSION["redirect"] != "")
      {
        $redirect = $_SESSION["redirect"];
        unset($_SESSION["redirect"]);
      }
      elseif(isset($_SERVER["HTTP_REFERER"]) && $_SERVER["HTTP_REFERER"] != "" && strpos($_SERVER["HTTP_REFERER"], $_SERVER["SERVER_NAME"]) !== false)
      {
        $redirect = $_SERVER["HTTP_REFERER"];
      }
      return $redirect;
    }//getRedirectUrl

		////
		// returns internal page info
		//
		function getPageInfo($where = "")
		{
			FJF_BASE::log(__METHOD__);

			$sql = "";
			$sql .= "SELECT p.id, pr.title, pr.description, pr.meta_title, pr.meta_keywords, pr.meta_description";
			$sql .= " FROM cms_content_revisions pr";
			$sql .= " LEFT JOIN cms_content p ON pr.item_id=p.id";
			$sql .= " WHERE p.current_id=pr.id AND p.status=1 AND pr.published=1";
			$sql .= " AND (pr.use_date_publish=0 OR pr.date_publish<'CURRENT_DATE')";
			$sql .= " AND (pr.use_date_expiration=0 OR pr.date_expiration>'CURRENT_DATE')";
			$sql .= " AND (p.need_approval=0 OR pr.approval=1)";
			$sql .= $where ? " AND (" . $where . ")" : "";
			$sql .= " LIMIT 1";

			$params["CURRENT_DATE"] = date('Y-m-d H:i:s');

			$record = $this->selectRecords($sql, $params);
			$result = (is_array($record) && is_object($record[0])) ? $record[0] : null;

			return $result;
		}//getPageInfo()

		function savePost()
		{
			$r = urlencode(FJF_BASE::encrypt(serialize(array("post" => $_POST, "url" => $_SERVER["REQUEST_URI"])), "request"));
			setcookie("REQUEST_OBJ", $r, time()+86400, "/");
		}//savePost

		function doPost()
		{
			$r = isset($_COOKIE["REQUEST_OBJ"]) && $_COOKIE["REQUEST_OBJ"] ? @unserialize(FJF_BASE::decrypt(urldecode($_COOKIE["REQUEST_OBJ"]), "request")) : null;
			if(!is_array($r)) return;
			setcookie("REQUEST_OBJ", "", time()+86400, "/");
			unset($_COOKIE["REQUEST_OBJ"]);

			$post = isset($r["post"]) ? $r["post"] : false;
			$url = isset($r["url"]) ? $r["url"] : "";

	    $html = '<html><head></head><body><form name="post_form" method="post" action="' . $url . '">';
			foreach($post as $k => $v)
			{
				if(!is_array($v)) $html .= '<input type="hidden" name="'.$k.'" value="'.$v.'">';
				else foreach($v as $k2 => $v2) $html .= '<input type="hidden" name="'.$k.'['.$k2.']" value="'.$v2.'">';
			}
			$html .= '</form><script>document.post_form.submit();</script></body></html>';
			exit($html);
		}//doPost

		function getSiteVar($name)
		{
			return ($result = $this->getRecordBy("site_vars", $name, "name")) ? ($result->type == "serialize" ? unserialize($result->value) : stripslashes($result->value)) : null;
		}//getSiteVar()

		function getSiteVars()
		{
			$result = array();
			if($tmp = $this->getRecords("site_vars")){
				foreach($tmp as $i){
					$result[$i->name] = $i->type == "serialize" ? unserialize($i->value) : $i->value;
				}
			}
			return $result;
		}//getSiteVars()

		function setSiteVar($name, $value)
		{
			if(is_array($value))
			{
				$value = serialize(array_map("stripslashes", $value));
				$data["type"] = "serialize";
			}
			$data["name"] = $name;
			$data["value"] = $value;
			return $this->saveRecord("site_vars", $data, "name");
		}//setSiteVar()

		function getSiteMenus($category)
		{
			$result = array();

			$result["main"] = $this->getRecords("cms_content_categories", "parent_id=0 AND status=1 ORDER BY position");
			$result["submenu"] = null;

			if (is_array($result["main"]))
			{
				foreach ($result["main"] as $res)
				{
					if ($res->url_title == $category)
					{
						$result["category_title"] = ($res->full_name != "") ? $res->full_name : $res->name;
						$result["submenu"] = $this->getRecords("cms_content_categories", "parent_id=" . $res->id . " AND status=1 ORDER BY position");
					}
				}
			}

			return $result;
		}//getSiteVars()

		function lock($table, $id){
			$this->saveRecord($table, array(
				"id" => $id,
				"is_locked" => $this->uid,
				"locked_time" => date("Y-m-d H:i:s")
			));
		}//lock

		function unlock($table, $id){
			$this->saveRecord($table, array(
				"id" => $id,
				"is_locked" => 0
			));
		}//unlock
		
		
		function checkUserStocks(){
		  if(!$this->uid) return false;
		  $prams['USER_ID'] = $this->uid;
      $result = $this->countFrom("users_stocks", "user_id='USER_ID'", $prams);
      return $result;
		}
		
		function checkLockedPage($redirect = ""){
			$result = false;
			include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/pricing_model.php");
			$tier = $this->isLoggedUser() ? $_SESSION["user"]->tier : 0;
			$result = PricingModel::isLockedPage($tier, $GLOBALS["REQUEST_CONTAINER"]["cmd"]);
			if($result !== false){
				$redirect = $redirect ? $redirect : $result;
				if($redirect) FJF_BASE::redirect($redirect);
				$result = true;
			}
			return $result;
		}
		
		function checkTierLevel($redirect = "/"){
			if($this->isLoggedUser()){
				$user = FJF_BASE_RICH::getRecords("users", "id = 'ID_VAL'", array("ID_VAL" => $this->uid), true, 'tier, customer_id, subscribe_id');
				if($user && $user->tier != $_SESSION["user"]->tier){
					include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/user_model.php");
					$_SESSION["user"]->tier = $user->tier;
					$_SESSION["user"]->customer_id = $user->customer_id;
					$_SESSION["user"]->subscribe_id = $user->subscribe_id;
					$userModel = new UserModel();
					$userModel->changePendingStatus($this->uid, $user->tier);
					if($user->tier == 0) FJF_BASE::redirect($redirect);
					return true;
				}
			}
			return false;
		}
		
		function getSitemapArticles(){
		  FJF_BASE::log(__METHOD__);
      $sql = "SELECT a.title, a.url_title, ac.url_title cat_url_title";
      $sql .= " FROM articles a LEFT JOIN articles_categories ac ON FIND_IN_SET(ac.id, a.categories_ids)";
      $sql .= " WHERE a.status=1 AND a.published=1 AND a.datetime<=NOW()";
      $sql .= " GROUP BY a.id";
      $sql .= " ORDER BY a.title";
      return $this->selectRecords($sql, null);
		}

	}

?>