<?php

class XigniteModel{
	
	static function get($api, array $parameteres = array(), $query = "", $skipValidation = false){
		include_once($GLOBALS["WEB_APPLICATION_CONFIG"]["models_path"] . "/rest_model.php");
		
		static $rest;
		if(!isset($rest)) $rest = new RestModel;
		else $rest->flush();
		
		$api = explode(":", $api);
		if(count($api) == 1){
			$api = $api[0];
			$apiHost = "www";
		}else{
			$apiHost = $api[0];
			$api = $api[1];
		}
		
		$api = explode(".", $api);
		$contentType = count($api) < 3 ? "json" : $api[2];
		$apiCategory = $api[0];
		$apiMethod = $api[1];
		
		$parameteres["_Token"] = $GLOBALS["WEB_APPLICATION_CONFIG"]["xignite_api_token"];
		$parameteres = http_build_query($parameteres);
		$parameteres .= ($parameteres ? "&" : "") . $query;
		$url = "http://" . $apiHost . ".xignite.com/" . $apiCategory . "." . $contentType . "/" . $apiMethod . ($parameteres ? "?" . $parameteres : "");
		$rest->setUrl($url);
		$rest->execute();
		$info = $rest->getResponseInfo();
		if($info && isset($info["http_code"]) && $info["http_code"] == 200){
			$response = $rest->getResponseBody();
			XigniteModel::saveLog($url,$response);
			if($contentType == "json"){
				$response = $response ? json_decode($response) : null;
				if($skipValidation) return $response;
				if($response && isset($response->Outcome) && $response->Outcome == "Success"
				|| is_array($response) && isset($response[0]->Outcome) && $response[0]->Outcome == "Success") return $response;
			}elseif($contentType == "asmx"){
				$doc = new DOMDocument();
				$doc->loadXML($response);
				if($skipValidation) return $doc;
				$outcome = $doc->getElementsByTagName("Outcome");
				return $outcome->item(0)->nodeValue == "Success" ? $doc : null;
			}else{
				return $response;
			}
		}
		
		return null;
	}//get
	
	static function saveLog($query,$response){
		return true;//dismiss log
		$file = $GLOBALS["WEB_APPLICATION_CONFIG"]["app_data_path"].'/xignite_log';
		if(file_exists($file) && is_writable($file)){
			$file = $file.'/log_'.date("Y_m_d");
			$log = date("Y-m-d H:i:s")."\n";
			$log .= 'Query: '.$query."\n";
			$log .= 'Response: '.$response."\n";
			if($fp = fopen($file, "a+")){
				$res = fwrite($fp, $log);
				fclose($fp);
			}
		}
		return true;
	}
	
	static function getQuote($ticker, $apiName){
		if($apiName == 'BATSRealtime') return XigniteModel::get("nasdaqlastsale:xNASDAQLastSale.GetLastSale", array(
			"Symbol" => $ticker
		));
		if($apiName == 'GlobalDelayed') return XigniteModel::get("globalquotes:xGlobalQuotes.GetGlobalDelayedQuote", array(
			"Identifier" => $ticker,
			"IdentifierType" => "Symbol"
		));
		return null;
	}//getQuote
	
	static function getQuotes($tickers, $apiName = null){
		$tmpTickers = array_values($tickers);
		if($tickers && is_object(array_shift($tmpTickers))){
			$apis = array();
			foreach($tickers as $stock){
				if(!$stock->API_NAME) continue;
				if(!array_key_exists($stock->API_NAME, $apis)) $apis[$stock->API_NAME] = array();
				$apis[$stock->API_NAME][] = $stock->PRICING_TICKER;
			}
			$result = array();
			foreach($apis as $apiName => $tickers) $result = array_merge($result, XigniteModel::getQuotes($tickers, $apiName));
			return $result;
		}
		
		if($apiName == 'BATSRealtime'){
			
			if($data = XigniteModel::get("nasdaqlastsale:xNASDAQLastSale.GetLastSales", array(
				"Symbols" => implode(",", $tickers),
				"_fields" => "Symbol,Outcome,Last,PercentChange,PreviousClose,Date,Time"
			), null, true)){
				for($i = 0, $l = count($data); $i < $l; $i++) if($data[$i]->Outcome != 'Success') unset($data[$i]);
				return array_values($data);
			}
			
		}elseif($apiName == 'GlobalDelayed'){
			
			if($data = XigniteModel::get("globalquotes:xGlobalQuotes.GetGlobalDelayedQuotes", array(
				"Identifiers" => implode(",", $tickers),
				"IdentifierType" => "Symbol",
				"_fields" => "Outcome,Security.Symbol,Last,PercentChange,PreviousClose,Date,Time"
			), null, true)){
				
				for($i = 0, $l = count($data); $i < $l; $i++){
					if($data[$i]->Outcome != 'Success'){
						unset($data[$i]);
					}else{
						$data[$i]->Symbol = $data[$i]->Security->Symbol;
						unset($data[$i]->Security);
					}
				}
				return array_values($data);
				
			}
			
		}
		return array();
	}//getQuotes
	
	static function getTicksMinTime(){
		return strtotime("today 8:00");
	}//getTicksMinTime
	
	static function getTicksMaxTime(){
		return strtotime("today 20:00");
	}//getTicksMaxTime
	
	static function getTicks($ticker, $apiName, $startTime, $endTime, $precision = 'Minute'){
		$records = array();
		$tmp = null;
		$startTime = date("m/d/Y H:i", max($startTime, XigniteModel::getTicksMinTime()));
		$endTime = date("m/d/Y H:i", min($endTime, XigniteModel::getTicksMaxTime()));
		
		switch($apiName){
			case 'BATSRealtime':
				
				$tmp = XigniteModel::get("nasdaqlastsale:xNASDAQLastSale.GetBars.json", array(
					"Symbol" => $ticker,
					"StartTime" => $startTime,
					"EndTime" => $endTime,
					"Precision" => $precision.'s',
					"Period" => 1,
					"_fields" => "Bars.EndDate,Bars.EndTime,Bars.Close"
				), null, true);
				$field = "Close";
				$data = "Bars";
				
				break;
			case 'GlobalDelayed':
				
				$tmp = XigniteModel::get("globalquotes:v3/xGlobalQuotes.GetBars.json", array(
					"Identifier" => $ticker,
					"IdentifierType" => "Symbol",
					"StartTime" => $startTime,
					"EndTime" => $endTime,
					"Precision" => $precision.'s',
					"Period" => 1,
					"_fields" => "Bars.EndDate,Bars.EndTime,Bars.Close"
				), null, true);
				$field = "Close";
				$data = "Bars";
				
				break;
		}
		$timeFormat = ($precision == 'Second') ? ":i:s" : ":i:00";
		if($tmp && isset($tmp->{$data})){
			foreach($tmp->{$data} as $item){
				$time = strtotime($item->EndDate . " " . $item->EndTime);
				$h = date("G", $time);
				if(intval($h, 10) < 8) $h += 12;
				if($h < 10) $h = '0' . $h;
				$records[date("Y-m-d " . $h . $timeFormat, $time)] = floatval($item->{$field});
			}
		}
		unset($tmp);
		
		return $records;
	}//getTicks
	
	static function getHistoricalQuotes($ticker, $startDate, $endDate){
		$records = array();
		$startDate = date("m/d/Y", max(strtotime('-5 years'), $startDate));
		$endDate = date("m/d/Y", min(time(), $endDate));
		if ($startDate == date("m/d/Y")) return $records;
		$tmp = XigniteModel::get("xGlobalHistorical.GetGlobalHistoricalQuotesRange.json", array(
			"Identifier" => $ticker,
			"IdentifierType" => 'Symbol',
			"StartDate" => $startDate,
			"EndDate" => $endDate,
			"AdjustmentMethod" => 'SplitOnly',
			"_fields" => "GlobalQuotes.Date,GlobalQuotes.Last"
		), null, true);
		$field = "Last";
		$data = "GlobalQuotes";
		if($tmp && isset($tmp->{$data})){
			foreach($tmp->{$data} as $item){
				$time = strtotime($item->Date);
				$records[date("Y-m-d 00:00:00", $time)] = floatval($item->{$field});
			}
		}
		return $records;
	}//getHistoricalQuotes
	
}

?>