function heatmap(treeData){
	if(!treeData) return;
	
	var animate;
	
	(function(){
		var labelType, useGradients, nativeTextSupport, ua = navigator.userAgent,
	      iStuff = ua.match(/iPhone/i) || ua.match(/iPad/i),
	      typeOfCanvas = typeof HTMLCanvasElement,
	      nativeCanvasSupport = (typeOfCanvas == 'object' || typeOfCanvas == 'function'),
	      textSupport = nativeCanvasSupport 
	        && (typeof document.createElement('canvas').getContext('2d').fillText == 'function');
	  //I'm setting this based on the fact that ExCanvas provides text support for IE
	  //and that as of today iPhone/iPad current text support is lame
	  labelType = (!nativeCanvasSupport || (textSupport && !iStuff))? 'Native' : 'HTML';
	  nativeTextSupport = labelType == 'Native';
	  useGradients = nativeCanvasSupport;
	  //animate = !(iStuff || !nativeCanvasSupport);
	  animate = false;
	})();
	
	function getStockHtml(stock){
    var prefix = '', className = '';
    if(stock.CHANGE > 0){
    	prefix = '&#8593; ';
    	className = 'green';
    }
    else if(stock.CHANGE < 0){
    	prefix = '&#8595; ';
    	className = 'red';
    }
		var html = stock.TICKER + ' - ' + stock.COMPANY + '<br />';
    html += stock.EXCHANGE + '<br />';
    html += stock.PRICE;
    html += ' <span'+(className ? ' class="'+className+'"' : '')+'>' + prefix + parseFloat(stock.CHANGE).toFixed(2) + ' (' + stock.PERCENT_CHANGE + '%)</span><br />';
    if(stock.SENTIMENT > 0) className = 'green';
    else if(stock.SENTIMENT < 0) className = 'red';
    else className = '';
    html += 'Sentiment: <span'+(className ? ' class="'+className+'"' : '')+'>' + parseFloat(stock.SENTIMENT).toFixed(3) + '</span><br />';
    if(stock.BUZZ > 1) className = 'green';
    else if(stock.BUZZ < 1) className = 'red';
    else className = '';
    html += 'Buzz: <span'+(className ? ' class="'+className+'"' : '')+'>' + parseFloat(stock.BUZZ).toFixed(2) + '</span>';
    return html;
	}//getStockHtml
	
  //init TreeMap
  var tm = new $jit.TM.Squarified({
    injectInto: 'heatmap',
    titleHeight: 24,
    animate: animate,
    offset: 0,
    Events: {
      enable: true,
      onClick: function(node) {
        if(node){
        	if(node.data.type == 'stock') window.location.assign('/stock/'+encodeURIComponent(encodeURIComponent(node.data.stock.TICKER))+'/');
        	else if(node.data.type == 'group') tm.enter(node);
        }
      },
      onRightClick: function() {
        tm.out();
      }
    },
    duration: 1000,
    Tips: {
      enable: true,
      offsetX: 20,
      offsetY: 20,
      onShow: function(tip, node, isLeaf, domElement) {
      	if(node.data.type == 'stock') $(tip).html(getStockHtml(node.data.stock)).css('opacity', '').addClass('heatmap_tip');
      	else $(tip).empty().css('opacity', 0);
      }
    },
    onCreateLabel: function(domElement, node){
    	var box = $(domElement)
    	.addClass('type-'+node.data.type)
    	.html(node.name)
    	.hover(function(){
    		if(node.id == 'root') return;
    		$('#heatmap .node').removeClass('active');
    		box.addClass('active');
    	}, function(){
    		box.removeClass('active');
    	});
    }
  });
  
  var wrapper = $('#heatmap'), mode = wrapper.attr('data-mode'), exchange = wrapper.attr('data-exchange'), tickers = wrapper.attr('data-tickers');
  var timeout = null;
  
  function setData(data){
  	if(data){
	  	tm.loadJSON(data);
	  	tm.refresh();
	  }
	  update();
  }
  
  function getData(){
  	$.get('/ajax/heat-maps/', {
			action: 'get_tree',
			view_mode: wrapper.attr('data-view-mode'),
			mode: mode,
			exchange: exchange,
			tickers: tickers
		}, function(response){
			setData(response && response.tree ? response.tree : null);
			$('#dashboard-nowtime').html(response.time);
		}, 'json');
  }
  
  function update(){
  	if(timeout) clearTimeout(timeout);
  	timeout = setTimeout(getData, 60000);
  }
  
  setData(treeData);
  updateFunctions.push(function(){
  	$('.dashboard-view-mode').text('Currently Displaying: '+(VIEW_MODE == 'MP' ? 'Market Prophits' : 'Crowd'));
  	getData();
  });
}//heatmap

$(function(){
	
	var request, prev = '', val;
	var input = $('#heatmapExchangeSearch');
	var list = $('#heatmapExchangeSearchList').hover(function(){
		list.find('tr').removeClass('active');
	});
	
	function show(){
  	if(list.find('div').length){
  		$('body').bind('click', onclick);
  		list.fadeIn('fast');
  	}
	}
	
  function hide(){
  	$('body').unbind('click', onclick);
  	list.fadeOut('fast');
  }
  
  function onclick(event){
    if(!$(event.target).closest('#heatmapExchangeSearchWrapper').length) hide();
  }
  
	function move(step){
		if(!step) step = 1;
		var items = list.find('div'), total = items.length, current;
		if(total){
			current = items.filter('.active');
			current = items
				.removeClass('active')
				.eq(((current.length ? items.index(current) : (step > 0 ? -1 : total)) + total + step) % total)
				.addClass('active');
		}
	}
	
  function search(q){
  	if(q === '') return;
  	hide();
  	if(request) request.abort();
	  request = $.get('/ajax/heat-maps/', {action: 'search_exchange', q: q}, function(response){
    	request = null;
      if(!response || !response.records) return true;
     	list.empty();
     	$.each(response.records, function(){
     		list.append($('<div></div>').text(this));
     	});
     	show();
	  }, 'json');
  }
  
  list.find('div').live('click', function(){
  	window.location.assign((''+window.location).replace(/[\?\#].*$/, '') + '?exchange='+$(this).text());
  });
	
	input.focus(show).keyup(function(e){
		if(e.keyCode == 40) move(1);
		else if(e.keyCode == 38) move(-1);
		else if(e.keyCode == 13){
			var active = list.find('div.active');
			if(active.length) active.click();
		}else{
			$('#search-list tr.active').removeClass('active');
			val = input.val();
			if(val != prev){
				prev = val;
				search(val);
			}
		}
	});
	
});